@@ -1,11 +1,8 @@
 
 
 
-import re
 import os
 import sys
-import uuid
-import decimal
 import asyncio
 import inspect
 import traceback
@@ -16,16 +13,17 @@ from aioquic.asyncio.server import serve
 from aioquic.asyncio.protocol import QuicConnectionProtocol
 from aioquic.quic.configuration import QuicConfiguration
 from aioquic.quic.events import QuicEvent, StreamDataReceived
-from typing import Any, AsyncGenerator, Union, get_origin, get_args
+from typing import Any, AsyncGenerator, Union
 
 
 from gnobjects.net.objects import GNRequest, GNResponse, FileObject, CORSObject, TemplateObject
 from gnobjects.net.fastcommands import AllGNFastCommands, GNFastCommand
 
 
+from ._func_params_validation import register_schema_by_key, validate_params_by_key
+from ._cors_resolver import resolve_cors
 
-
-
+from ._routes import Route, _compile_path, _ensure_async, _convert_value
 
 
 
@@ -46,9 +44,9 @@ logger = logging.getLogger("GNServer")
 logger.setLevel(logging.DEBUG)
 
 console = logging.StreamHandler()
-console.setLevel(logging.INFO)
+console.setLevel(logging.DEBUG)
 console.setFormatter(logging.Formatter("[GNServer] %(name)s: %(levelname)s: %(message)s"))
-
+logger.addHandler(console)
 
 
 def guess_type(filename: str) -> str:
@@ -112,281 +110,11 @@ def guess_type(filename: str) -> str:
     return mime_map.get(ext, "application/octet-stream")
 
 
-import re
-from typing import List
-
-# regex для !{var}, поддерживает вложенность через точку
-TPL_VAR_RE = re.compile(r'(?<!\\)!\{([A-Za-z_][A-Za-z0-9_\.]*)\}')
-
-# список mime, которые считаем текстовыми
-TEXTUAL_MIME_PREFIXES = [
-    "text/",                       # text/html, text/css, text/plain
-]
-TEXTUAL_MIME_EXACT = {
-    "application/javascript",
-    "application/json",
-    "application/xml",
-    "application/xhtml+xml"
-}
-TEXTUAL_MIME_SUFFIXES = (
-    "+xml",  # например application/rss+xml
-    "+json", # application/ld+json
-)
-
-def extract_template_vars(filedata: bytes, mime: str) -> List[str]:
-    """
-    Ищет все !{var} в тексте, если MIME относится к текстовым.
-    """
-    mime = (mime or "").lower().strip()
-
-    # определяем, текстовый ли mime
-    is_textual = (
-        mime.startswith(tuple(TEXTUAL_MIME_PREFIXES))
-        or mime in TEXTUAL_MIME_EXACT
-        or mime.endswith(TEXTUAL_MIME_SUFFIXES)
-        or "javascript" in mime
-        or "json" in mime
-        or "xml" in mime
-    )
-
-    if not is_textual:
-        return []
-
-    try:
-        text = filedata.decode("utf-8", errors="ignore")
-    except Exception:
-        return []
-
-    return list(set(m.group(1) for m in TPL_VAR_RE.finditer(text)))
-
-
-import re
-from urllib.parse import urlparse
-
-def resolve_cors(origin_url: str, rules: list[str]) -> bool:
-    """
-    Возвращает origin_url если он матчится хотя бы с одним правилом.
-    Правила:
-      - "*.example.com"    -> wildcard (одна метка)
-      - "**.example.com"   -> globstar (0+ меток)
-      - "pages.*.core.gn"  -> смешанное
-      - "gn://*.site.tld" -> с проверкой схемы
-      - "!<regex>"         -> полное соответствие по regex к origin_url
-    """
-
-    if origin_url == 'gn:proxy:sys':
-        return True
 
 
 
 
 
-    origin = origin_url.rstrip("/")
-    pu = urlparse(origin)
-    scheme = (pu.scheme or "").lower()
-    host = (pu.hostname or "").lower()
-    port = pu.port  # может быть None
-
-    if not host:
-        return False
-
-    for rule in rules:
-        rule = rule.rstrip("/")
-
-        # 1) Регекс-правило
-        if rule.startswith("!"):
-            pattern = rule[1:]
-            if re.fullmatch(pattern, origin):
-                return True
-            continue
-
-        # 2) Разбор схемы/хоста в правиле
-        r_scheme = ""
-        r_host = ""
-        r_port = None
-
-        if "://" in rule:
-            pr = urlparse(rule)
-            r_scheme = (pr.scheme or "").lower()
-            # pr.netloc может содержать порт
-            netloc = pr.netloc.lower()
-            # разберём порт, если есть
-            if ":" in netloc and not netloc.endswith("]"):  # простая обработка IPv6 не требуется здесь
-                name, _, p = netloc.rpartition(":")
-                r_host = name
-                try:
-                    r_port = int(p)
-                except ValueError:
-                    r_port = None
-            else:
-                r_host = netloc
-        else:
-            r_host = rule.lower()
-
-        # схема в правиле задана -> должна совпасть
-        if r_scheme and r_scheme != scheme:
-            continue
-        # порт в правиле задан -> должен совпасть
-        if r_port is not None and r_port != port:
-            continue
-
-        # 3) Сопоставление хоста по шаблону с * и ** (по меткам)
-        if _host_matches_pattern(host, r_host):
-            return True
-
-    return False
-
-
-def _host_matches_pattern(host: str, pattern: str) -> bool:
-    """
-    Матчит host против pattern по доменным меткам:
-      - '*'  -> ровно одна метка
-      - '**' -> ноль или больше меток
-      Остальные метки — точное совпадение (без внутр. вайлдкардов).
-    Примеры:
-      host=pages.static.core.gn, pattern=**.core.gn -> True
-      host=pages.static.core.gn, pattern=pages.*.core.gn -> True
-      host=pages.static.core.gn, pattern=*.gn.gn -> False
-      host=abc.def.example.com, pattern=*.example.com -> False (нужно **.example.com)
-      host=abc.example.com,     pattern=*.example.com -> True
-    """
-    host_labels = host.split(".")
-    pat_labels = pattern.split(".")
-
-    # быстрый путь: точное совпадение без вайлдкардов
-    if "*" not in pattern:
-        return host == pattern
-
-    # рекурсивный матч с поддержкой ** (globstar)
-    def match(hi: int, pi: int) -> bool:
-        # оба дошли до конца
-        if pi == len(pat_labels) and hi == len(host_labels):
-            return True
-        # закончился паттерн — нет
-        if pi == len(pat_labels):
-            return False
-
-        token = pat_labels[pi]
-        if token == "**":
-            # два варианта:
-            #  - пропустить '**' (ноль меток)
-            if match(hi, pi + 1):
-                return True
-            #  - съесть одну метку (если есть) и остаться на '**'
-            if hi < len(host_labels) and match(hi + 1, pi):
-                return True
-            return False
-        elif token == "*":
-            # нужно съесть ровно одну метку
-            if hi < len(host_labels):
-                return match(hi + 1, pi + 1)
-            return False
-        else:
-            # точное совпадение метки
-            if hi < len(host_labels) and host_labels[hi] == token:
-                return match(hi + 1, pi + 1)
-            return False
-
-    return match(0, 0)
-
-
-
-
-@dataclass
-class Route:
-    method: str
-    path_expr: str
-    regex: Pattern[str]
-    param_types: dict[str, Callable[[str], Any]]
-    handler: Callable[..., Any]
-    name: str
-    cors: Optional[CORSObject]
-
-_PARAM_REGEX: dict[str, str] = {
-    "str":   r"[^/]+",
-    "path":  r".+",
-    "int":   r"\d+",
-    "float": r"[+-]?\d+(?:\.\d+)?",
-    "bool":  r"(?:true|false|1|0)",
-    "uuid":  r"[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-"
-             r"[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-"
-             r"[0-9a-fA-F]{12}",
-    "datetime": r"\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d+)?Z?",
-    "date":     r"\d{4}-\d{2}-\d{2}",
-    "time":     r"\d{2}:\d{2}:\d{2}(?:\.\d+)?",
-    "decimal":  r"[+-]?\d+(?:\.\d+)?",
-}
-
-_CONVERTER_FUNC: dict[str, Callable[[str], Any]] = {
-    "int":     int,
-    "float":   float,
-    "bool":    lambda s: s.lower() in {"1","true","yes","on"},
-    "uuid":    uuid.UUID,
-    "decimal": decimal.Decimal,
-    "datetime": datetime.datetime.fromisoformat,
-    "date":     datetime.date.fromisoformat,
-    "time":     datetime.time.fromisoformat,
-}
-
-def _compile_path(path: str) -> tuple[Pattern[str], dict[str, Callable[[str], Any]]]:
-    param_types: dict[str, Callable[[str], Any]] = {}
-    rx_parts: list[str] = ["^"]
-    i = 0
-    while i < len(path):
-        if path[i] != "{":
-            rx_parts.append(re.escape(path[i]))
-            i += 1
-            continue
-        j = path.index("}", i)
-        spec = path[i+1:j]
-        i = j + 1
-
-        if ":" in spec:
-            name, conv = spec.split(":", 1)
-        else:
-            name, conv = spec, "str"
-
-        if conv.startswith("^"):
-            rx = f"(?P<{name}>{conv})"
-            typ = str
-        else:
-            rx = f"(?P<{name}>{_PARAM_REGEX.get(conv, _PARAM_REGEX['str'])})"
-            typ = _CONVERTER_FUNC.get(conv, str)
-
-        rx_parts.append(rx)
-        param_types[name] = typ
-
-    rx_parts.append("$")
-    return re.compile("".join(rx_parts)), param_types
-
-def _convert_value(raw: str | list[str], ann: Any, fallback: Callable[[str], Any]) -> Any:
-    origin = get_origin(ann)
-    args   = get_args(ann)
-
-    if isinstance(raw, list) or origin is list:
-        subtype = args[0] if (origin is list and args) else str
-        if not isinstance(raw, list):
-            raw = [raw]
-        return [_convert_value(r, subtype, fallback) for r in raw]
-
-    # --- fix Union ---
-    if origin is Union:
-        for subtype in args:
-            try:
-                return _convert_value(raw, subtype, fallback)
-            except Exception:
-                continue
-        return raw  # если ни один тип не подошёл
-
-    conv = _CONVERTER_FUNC.get(ann, ann) if ann is not inspect._empty else fallback
-    return conv(raw) if callable(conv) else raw
-
-def _ensure_async(fn: Callable[..., Any]) -> Callable[..., Any]:
-    if inspect.iscoroutinefunction(fn) or inspect.isasyncgenfunction(fn):
-        return fn
-    async def wrapper(*args, **kw):
-        return fn(*args, **kw)
-    return wrapper
 
 class App:
     def __init__(self):
@@ -412,6 +140,7 @@ class App:
                     cors
                 )
             )
+            register_schema_by_key(fn)
             return fn
         return decorator
 
@@ -490,7 +219,7 @@ class App:
                     return GNResponse("gn:backend:801", {'error': 'Cors error. Route has cors but request has no origin url.'})
                 if not resolve_cors(request._origin, r.cors.allow_origins):
                     return GNResponse("gn:backend:802", {'error': 'Cors error: origin'})
-                if request.method not in r.cors.allow_methods and '*' not in r.cors.allow_methods:
+                if r.cors.allow_methods is not None and request.method not in r.cors.allow_methods and '*' not in r.cors.allow_methods:
                     return GNResponse("gn:backend:803", {'error': 'Cors error: method'})
 
             sig = inspect.signature(r.handler)
@@ -509,12 +238,18 @@ class App:
                 raw = qvals if len(qvals) > 1 else qvals[0]
                 kw[qn] = _convert_value(raw, _ann(qn), str)
 
-            if "request" in sig.parameters:
-                kw["request"] = request
             
             params = set(sig.parameters.keys())
             kw = {k: v for k, v in kw.items() if k in params}
 
+            
+            rv = validate_params_by_key(kw, r.handler)
+            if rv is not None:
+                raise AllGNFastCommands.UnprocessableEntity({'dev_error': rv, 'user_error': f'Server request error {self.domain}'})
+
+            if "request" in sig.parameters:
+                kw["request"] = request
+
             if inspect.isasyncgenfunction(r.handler):
                 return r.handler(**kw)
 
@@ -532,7 +267,7 @@ class App:
                         return GNResponse("gn:backend:801", {'error': 'Cors error. Route has cors but request has no origin url. [2]'})
                     if not resolve_cors(request._origin, result._cors.allow_origins):
                         return GNResponse("gn:backend:802", {'error': 'Cors error: origin'})
-                    if request.method not in result._cors.allow_methods and '*' not in result._cors.allow_methods:
+                    if result._cors.allow_methods is not None and request.method not in result._cors.allow_methods and '*' not in result._cors.allow_methods:
                         return GNResponse("gn:backend:803", {'error': 'Cors error: method'})
                 return result
             else:
@@ -579,7 +314,6 @@ class App:
     def _init_sys_routes(self):
         @self.post('/!gn-vm-host/ping', cors=CORSObject())
         async def r_ping(request: GNRequest):
-            
             if request.client.ip != '127.0.0.1':
                 raise AllGNFastCommands.Forbidden()
             return GNResponse('ok', {'time': datetime.datetime.now(datetime.UTC).isoformat()})
@@ -615,11 +349,6 @@ class App:
                 if not stream: # если не стрим, то ждем конец quic стрима и запускаем обработку ответа
                     if event.end_stream:
                         request = GNRequest.deserialize(buf, mode)
-                        # request.stream_id = event.stream_id
-                        # loop = asyncio.get_event_loop()
-                        # request.fut = loop.create_future()
-
-
                         request.stream_id = event.stream_id  # type: ignore
                         asyncio.create_task(self._handle_request(request, mode))
                         logger.debug(f'Отправлена задача разрешения пакета {request} route -> {request.route}')
@@ -660,9 +389,6 @@ class App:
                                 logger.debug(f'Закрываем стрим [{event.stream_id}]')
                                 return
 
-
-
-
                 queue.put_nowait(request)
 
                 # отдаем очередь в интерфейс
@@ -684,7 +410,6 @@ class App:
             request.client._data['remote_addr'] = self._quic._network_paths[0].addr
 
             try:
-                
                 response = await self._api.dispatchRequest(request)
 
                 if inspect.isasyncgen(response):
@@ -764,8 +489,6 @@ class App:
             if run is not None:
                 await run()
 
-            
-
 
             if wait:
                 await asyncio.Event().wait()
