@@ -1,3 +1,4 @@
+import os
 import time
 import httpx
 import asyncio
@@ -75,13 +76,56 @@ class GNExceptions:
                     
 
 from KeyisBTools import TTLDict
-from KeyisBTools.cryptography.sign import S1
+from KeyisBTools.cryptography.sign import s2
+from KeyisBTools.cryptography import m1
+from KeyisBTools.cryptography.bytes import hash, userFriendly
+from KeyisBTools.models.serialization import serialize
+from KeyisBTools.ranges.positionRange import in_range
 from gnobjects.net.objects import GNRequest, GNResponse, Url
 
 from ._crt import crt_client
 
 
-s1 = S1()
+
+_log_levels: dict[str, int] = {
+    "NOTSET": 0,
+    "DEBUG": 10,
+    "INFO": 20,
+    "WARNING": 30,
+    "ERROR": 40,
+    "CRITICAL": 50,
+    "notset": 0,
+    "debug": 10,
+    "info": 20,
+    "warning": 30,
+    "error": 40,
+    "critical": 50,
+}
+
+def devLog(request: GNRequest, point: str, level: int, log: str):
+    """
+- NOTSET	 0
+- DEBUG	     10
+- INFO	     20
+- WARNING	 30
+- ERROR	     40
+- CRITICAL   50
+    """
+    if request.transportObject.routeProtocol.dev:
+        if hasattr(request, '_devDataLog'):
+            if request._devDataLogLevel <= level:
+                if in_range(point, request._devDataRange):
+
+                    log = f'[{level}]({datetime.datetime.now()}): {log}'
+
+                    request._devDataLog.append(log)
+
+
+
+
+
+
+
 
 
 async def chain_async(first_item, rest: AsyncIterable) -> AsyncGenerator:
@@ -92,7 +136,7 @@ async def chain_async(first_item, rest: AsyncIterable) -> AsyncGenerator:
 
 
 class AsyncClient:
-    def __init__(self):
+    def __init__(self, server_key: Optional[Union[bytes, str]] = None):
         self.__dns_core__ipv4 = '51.250.85.38:52943'
         self.__dns_gn__ipv4: Optional[str] = None
 
@@ -105,11 +149,25 @@ class AsyncClient:
         self.__dns_client: Optional[AsyncClient] = None
         self._dns_cache: TTLDict = TTLDict()
 
-    async def getCoreDNS(self, domain: str, token: Optional[bytes] = None, use_cache: bool = True, keep_alive: bool = False) -> str:
+        if server_key:
+            if isinstance(server_key, bytes):
+                self.__server_key = server_key
+            else:
+                if os.path.exists(server_key):
+                    self.__server_key = open(server_key, 'rb').read()
+                else:
+                    self.__server_key = userFriendly.decode(server_key)
+        else:
+            self.__server_key = None
+
+    async def getCoreDNS(self, domain: str, use_cache: bool = True, keep_alive: bool = False) -> str:
         if use_cache:
             resuilt = self._dns_cache.get(domain)
             if resuilt is not None:
                 return resuilt
+            
+        if domain == 'api.dns.core':
+            return self.__dns_core__ipv4
 
         if ':' in domain and domain.split('.')[-1].split(':')[0].isdigit() and domain.split(':')[-1].isdigit():
             return domain
@@ -118,8 +176,10 @@ class AsyncClient:
             if self.__dns_client is None:
                 self.__dns_client = AsyncClient()
 
-            if token is not None:
-                payload = {'sign': {'alg': 'KeyisB-c-s-m1', 'data': s1.sign(token)}}
+            if self.__server_key is not None:
+                s = s2.sign(self.__server_key)
+                data = m1.encrypt(s, domain.encode(), serialize({'domain': domain}), hash(self.__server_key))
+                payload = {'sign': {'alg': 'KeyisB-c-s-m1', 'data': s}, 'data': data}
             else:
                 payload = None
 
@@ -144,7 +204,7 @@ class AsyncClient:
         except:
             raise GNExceptions.ConnectionError.dns.connection
 
-    async def getGNDNS(self, domain: str, token: Optional[bytes] = None, use_cache: bool = True, keep_alive: bool = False) -> str:
+    async def getGNDNS(self, domain: str, use_cache: bool = True, keep_alive: bool = False) -> str:
         if use_cache:
             resuilt = self._dns_cache.get(domain)
             if resuilt is not None:
@@ -160,8 +220,8 @@ class AsyncClient:
             if self.__dns_client is None:
                 self.__dns_client = AsyncClient()
             
-            if token is not None:
-                payload = {'sign': {'alg': 'KeyisB-c-s-m1', 'data': s1.sign(token)}}
+            if self.__server_key is not None:
+                payload = {'sign': {'alg': 'KeyisB-c-s-m1', 'data': s1.sign(self.__server_key)}}
             else:
                 payload = None
 
@@ -187,11 +247,11 @@ class AsyncClient:
         except:
             raise GNExceptions.ConnectionError.dns.connection
 
-    async def getDNS(self, domain: str, token: Optional[bytes] = None, use_cache: bool = True, keep_alive: bool = False):
+    async def getDNS(self, domain: str, use_cache: bool = True, keep_alive: bool = False):
         if domain.endswith(('.core', '.gw', '.gn', '.cdn', '.sys', '.gwis', '.abs')):
             return await self.getCoreDNS(domain=domain, use_cache=use_cache, keep_alive=keep_alive)
         else:
-            return await self.getGNDNS(domain=domain, token=token, use_cache=use_cache, keep_alive=keep_alive)
+            return await self.getGNDNS(domain=domain, use_cache=use_cache, keep_alive=keep_alive)
 
     def addRequestCallback(self, callback: Callable, name: str):
         self.__request_callbacks[name] = callback
@@ -200,16 +260,17 @@ class AsyncClient:
         self.__response_callbacks[name] = callback
 
   
-    async def connect(self, domain: str, restart_connection: bool = False, reconnect_wait: float = 10, keep_alive: bool = True) -> 'QuicClient':
-        print('Запрос подключения')
+    async def connect(self, request: GNRequest, restart_connection: bool = False, reconnect_wait: float = 10, keep_alive: bool = True) -> 'QuicClient':
+        domain = request.url.hostname
+        devLog(request, '1.2.1', 10, f'Connect to {domain}: restart_connection={restart_connection}, reconnect_wait={reconnect_wait}, keep_alive={keep_alive}')
         if not restart_connection and domain in self._active_connections:
             c = self._active_connections[domain]
-            print(f'Подключение уже было [{c.status}]')
+            devLog(request, '1.2.2', 10, f'Connection to {domain} found: status={c.status}')
             if c.status == 'connecting':
-                print('ждем поделючения')
+                devLog(request, '1.2.3', 10, f'Waiting for connection to {domain}...')
                 try:
                     await asyncio.wait_for(c.connect_future, reconnect_wait)
-                    print('дождались')
+                    devLog(request, '1.2.4', 10, f'Connected to {domain}: status={c.status}')
                     if c.status == 'active':
                         return c
                     elif c.status == 'connecting': # если очень долго подключаемся, то кидаем ошибку
@@ -218,7 +279,7 @@ class AsyncClient:
                     elif c.status == 'disconnect':
                         raise GNExceptions.ConnectionError.client.connection
                 except:
-                    print('Заново соеденяемся...')
+                    devLog(request, '1.2.5', 10, f'Connection to {domain} failed or timed out.')
                     await self.disconnect(domain)
                         
             else:
@@ -228,7 +289,9 @@ class AsyncClient:
         c.status = 'connecting'
         self._active_connections[domain] = c
 
+        devLog(request, '1.2.6', 10, f'Connecting to {domain}...')
         data = await self.getDNS(domain)
+        devLog(request, '1.2.7', 10, f'Got DNS for {domain}: {data}')
 
         data = data.split(':')
 
@@ -239,7 +302,7 @@ class AsyncClient:
 
         c._disconnect_signal = f # type: ignore
         c._domain = domain # type: ignore
-
+        devLog(request, '1.2.8', 10, f'Creating QuicClient for {domain}...')
         await c.connect(data[0], int(data[1]), keep_alive=keep_alive)
         await c.connect_future
 
@@ -255,13 +318,31 @@ class AsyncClient:
     def _return_token(self, bigToken: str, s: bool = True) -> str:
         return bigToken[:128] if s else bigToken[128:]
 
+    async def _resolve_requests_transport(self, request: GNRequest) -> GNRequest:
+        
+        if request.transportObject.routeProtocol.dev:
+            if request.cookies is not None:
+                data: Optional[dict] = request.cookies.get('gn', {}).get('request', {}).get('transport', {}).get('::dev')
+                if data is not None:
+                    data['params']['logs']['data'] = []
+                    data['params']['data']['data'] = {}
+                    request._devDataLog = data['params']['logs']['data']
+                    request._devDataLogLevel = _log_levels[data['params']['logs']['data']]
+                    request._devData = data['params']['data']['data']
+                    request._devDataRange = data['params']['range']
+
+        return request
 
     async def request(self, request: Union[GNRequest, AsyncGenerator[GNRequest, Any]], keep_alive: bool = True, restart_connection: bool = False, reconnect_wait: float = 10) -> GNResponse:
-        if isinstance(request, GNRequest):
-            
+        devLog(request, '1.1.1', 10, f'Request starting... {request}: keep_alive={keep_alive}, restart_connection={restart_connection}, reconnect_wait={reconnect_wait}')
 
-            c = await self.connect(request.url.hostname, restart_connection, reconnect_wait, keep_alive=keep_alive)
-                
+        if isinstance(request, GNRequest):
+            devLog(request, '1.1.2', 10, 'Request is GNRequest')
+            
+            request = await self._resolve_requests_transport(request)
+
+            c = await self.connect(request, restart_connection, reconnect_wait, keep_alive=keep_alive)
+            devLog(request, '1.1.3', 10, f'Connected: {c.status}')
 
 
             for f in self.__request_callbacks.values():
@@ -275,6 +356,7 @@ class AsyncClient:
             return r
         
         else:
+
             c: Optional[QuicClient] = None
 
             async def wrapped(request) -> AsyncGenerator[GNRequest, None]:
@@ -288,7 +370,7 @@ class AsyncClient:
 
                     nonlocal c
                     if c is None:  # инициализируем при первом req
-                        c = await self.connect(request.url.hostname, restart_connection, reconnect_wait, keep_alive=keep_alive)
+                        c = await self.connect(request, restart_connection, reconnect_wait, keep_alive=keep_alive)
 
                     yield req
 
@@ -540,6 +622,7 @@ class RawQuicClient(QuicConnectionProtocol):
         if isinstance(request, GNRequest):
             blob = request.serialize(2)
             sid = self._quic.get_next_available_stream_id()
+            devLog(request, '1.3.1', 10, f'Sending request on stream {sid}')
             self._enqueue(sid, blob, True, False)
             self._schedule_flush()
 
@@ -700,7 +783,9 @@ class QuicClient:
             raise RuntimeError("Not connected")
         
         resp = await self._quik_core.request(request)
-        return GNResponse.deserialize(resp, 2)
+        r = GNResponse.deserialize(resp, 2)
+        devLog(request, '1.4.1', 10, f'Response received: {r}')
+        return r
 
     async def asyncRequestStream(self, request: Union[GNRequest, AsyncGenerator[GNRequest, Any]]) -> AsyncGenerator[GNResponse, None]:
         
